import subprocess

_dev_null = open('/dev/null', 'w')

def quiet_check_call(*args, **kwargs):
    return _quiet_subprocess_eval(subprocess.check_call, args, kwargs)

def quiet_call(*args, **kwargs):
    return _quiet_subprocess_eval(subprocess.call, args, kwargs)
    
def quiet_check_output(*args, **kwargs):
    return _check_output(*args, stderr=_dev_null, **kwargs)
    
def _quiet_subprocess_eval(func, args, kwargs):
    return func(*args, stdout=_dev_null, stderr=subprocess.STDOUT, **kwargs)

def _check_output(*popenargs, **kwargs):
    r"""Run command with arguments and return its output as a byte string.

Backported from Python 2.7 as it's implemented as pure python on stdlib.
"""
    process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)
    output, unused_err = process.communicate()
    retcode = process.poll()
    if retcode:
        cmd = kwargs.get("args")
        if cmd is None:
            cmd = popenargs[0]
        error = subprocess.CalledProcessError(retcode, cmd)
        error.output = output
        raise error
    return output



import os
import subprocess
import sys

import blah.repositories
import blah.fetcher

class WhatIsThisCommand(object):
    def create_parser(self, subparser):
        subparser.add_argument("directory", nargs="?")
    
    def execute(self, args):
        directory = args.directory if args.directory is not None else os.getcwd()
        repository = blah.repositories.find_repository(directory)
        if repository is None:
            print "Could not find source control repository"
        else:
            print "{0}+file://{1}".format(repository.type, repository.working_directory)

what_is_this_command = WhatIsThisCommand()

class FetchCommand(object):
    def create_parser(self, subparser):
        subparser.add_argument("repository_uri", metavar="repository-uri")
        subparser.add_argument("local_path", metavar="local-path")
    
    def execute(self, args):
        blah.fetcher.fetch(args.repository_uri, args.local_path)

commands = {
    "whatisthis": what_is_this_command,
    "what-is-this": what_is_this_command,
    "fetch": FetchCommand()
}



import os

from blah.util import quiet_check_call, quiet_call, quiet_check_output
import blah.files

class Git(object):
    name = "git"
    directory_name = ".git"
    default_branch = "origin/master"
    
    def clone(self, repository_uri, local_path):
        quiet_check_call(_command("clone", repository_uri, local_path))
        return GitRepository(local_path)
        
    def local_repo(self, working_directory):
        return GitRepository(working_directory)

class GitRepository(object):
    type = Git.name
    
    def __init__(self, working_directory):
        self.working_directory = working_directory
    
    def update(self):
        quiet_check_call(_command("fetch"), cwd=self.working_directory)

    def checkout_revision(self, revision):
        if quiet_call(_command("branch", "-r", "--contains", "origin/" + revision), cwd=self.working_directory) == 0:
            revision = "origin/" + revision
            
        quiet_check_call(_command("checkout", revision), cwd=self.working_directory)

    def remote_repo_uri(self):
        return quiet_check_output(_command("config", "remote.origin.url"), cwd=self.working_directory).strip()
        
    def head_revision(self):
        return quiet_check_output(_command("rev-parse", "HEAD"), cwd=self.working_directory).strip()

def _command(command, *args):
    return ["git", command] + list(args)



import os

import blah.systems
import blah.uri_parser

def fetch(uri_str, local_path, systems=None):
    if systems is None:
        systems = blah.systems.all_systems
    
    uri = blah.uri_parser.parse(uri_str)
    
    for vcs in systems:
        if uri.vcs == vcs.name:
            return fetch_with_vcs(uri, local_path, vcs)
            
    raise RuntimeError("Source control system not recognised: " + uri.vcs)
        
def fetch_with_vcs(uri, local_path, vcs):
    repository_uri = uri.repo_uri
    
    if os.path.exists(local_path):
        vcs_directory = os.path.join(local_path, vcs.directory_name)
        if not os.path.isdir(local_path):
            raise RuntimeError("Checkout path already exists, and is not directory: " + local_path)
        elif not os.path.isdir(vcs_directory):
            raise RuntimeError("VCS directory doesn't exist: " + vcs_directory)
        else:
            local_repo = vcs.local_repo(local_path)
            current_remote_uri = local_repo.remote_repo_uri()
            if current_remote_uri == repository_uri:
                local_repo.update()
            else:
                raise RuntimeError(
                    "Checkout directory is checkout of different URI: " + current_remote_uri +
                    "\nExpected: " + repository_uri
                )
    else:
        local_repo = vcs.clone(repository_uri, local_path)
        
    if uri.revision is None:
        revision = vcs.default_branch
    else:
        revision = uri.revision
        
    local_repo.checkout_revision(revision)
